@@ -12,7 +12,7 @@ use bls12_381::{G1Affine, G1Projective, G2Affine, G2Projective, Scalar};
 use group::GroupEncoding;
 use sha2::Sha256;
 use subtle::ConstantTimeEq;
-use zeroize::Zeroizing;
+use zeroize::{Zeroize, Zeroizing};
 
 use crate::generic_array::{
     typenum::{self, Unsigned, U144, U32, U48, U96},
@@ -29,16 +29,17 @@ use crate::{
 };
 
 /// The 'kty' value of a BLS key JWK
-pub const JWK_KEY_TYPE: &'static str = "EC";
+pub const JWK_KEY_TYPE: &'static str = "OKP";
 
 /// A BLS12-381 key pair
-#[derive(Clone)]
+#[derive(Clone, Zeroize)]
 pub struct BlsKeyPair<Pk: BlsPublicKeyType> {
     secret: Option<BlsSecretKey>,
     public: Pk::Buffer,
 }
 
 impl<Pk: BlsPublicKeyType> BlsKeyPair<Pk> {
+    /// Generate a new BLS key from a seed according to the KeyGen algorithm
     pub fn from_seed(seed: &[u8]) -> Result<Self, Error> {
         Ok(Self::from_secret_key(BlsSecretKey::generate(
             BlsKeyGen::new(seed)?,
@@ -61,6 +62,16 @@ impl<Pk: BlsPublicKeyType> BlsKeyPair<Pk> {
             Err(err_msg!(InvalidKeyData, "invalid BLS keypair"))
         }
     }
+
+    /// Accessor for the associated public key
+    pub fn bls_public_key(&self) -> &Pk::Buffer {
+        &self.public
+    }
+
+    /// Accessor for the associated secret key value, if any
+    pub fn bls_secret_scalar(&self) -> Option<&Scalar> {
+        self.secret.as_ref().map(|s| &s.0)
+    }
 }
 
 impl<Pk: BlsPublicKeyType> Debug for BlsKeyPair<Pk> {
@@ -145,7 +156,7 @@ where
 }
 
 impl<Pk: BlsPublicKeyType> ToJwk for BlsKeyPair<Pk> {
-    fn encode_jwk(&self, enc: &mut JwkEncoder<'_>) -> Result<(), Error> {
+    fn encode_jwk(&self, enc: &mut dyn JwkEncoder) -> Result<(), Error> {
         enc.add_str("crv", Pk::get_jwk_curve(enc.alg()))?;
         enc.add_str("kty", JWK_KEY_TYPE)?;
         Pk::with_bytes(&self.public, enc.alg(), |buf| enc.add_as_base64("x", buf))?;
@@ -164,6 +175,12 @@ impl<Pk: BlsPublicKeyType> ToJwk for BlsKeyPair<Pk> {
 
 impl<Pk: BlsPublicKeyType> FromJwk for BlsKeyPair<Pk> {
     fn from_jwk_parts(jwk: JwkParts<'_>) -> Result<Self, Error> {
+        if jwk.kty != JWK_KEY_TYPE {
+            return Err(err_msg!(InvalidKeyData, "Unsupported key type"));
+        }
+        if jwk.crv != Pk::JWK_CURVE {
+            return Err(err_msg!(InvalidKeyData, "Unsupported key algorithm"));
+        }
         ArrayKey::<Pk::BufferSize>::temp(|pk_arr| {
             if jwk.x.decode_base64(pk_arr)? != pk_arr.len() {
                 Err(err_msg!(InvalidKeyData))
@@ -190,7 +207,7 @@ impl<Pk: BlsPublicKeyType> FromJwk for BlsKeyPair<Pk> {
     }
 }
 
-#[derive(Clone, Debug, PartialEq, Eq)]
+#[derive(Clone, Debug, PartialEq, Eq, Zeroize)]
 #[repr(transparent)]
 pub(crate) struct BlsSecretKey(Scalar);
 
@@ -214,13 +231,22 @@ impl BlsSecretKey {
     }
 }
 
-// bls-signatures draft 4 version (incompatible with earlier)
+impl Drop for BlsSecretKey {
+    fn drop(&mut self) {
+        self.zeroize();
+    }
+}
+
+/// A key material generator compatible with KeyGen from the
+/// bls-signatures RFC draft 4 (incompatible with earlier)
+#[derive(Debug, Clone)]
 pub struct BlsKeyGen<'g> {
     salt: Option<GenericArray<u8, U32>>,
     ikm: &'g [u8],
 }
 
 impl<'g> BlsKeyGen<'g> {
+    /// Construct a new `BlsKeyGen` from a seed value
     pub fn new(ikm: &'g [u8]) -> Result<Self, Error> {
         if ikm.len() < 32 {
             return Err(err_msg!(Usage, "Insufficient length for seed"));
@@ -235,7 +261,7 @@ impl KeyMaterial for BlsKeyGen<'_> {
 
         self.salt.replace(match self.salt {
             None => Sha256::digest(SALT),
-            Some(salt) => Sha256::digest(salt.as_ref()),
+            Some(salt) => Sha256::digest(salt),
         });
         let mut extract = hkdf::HkdfExtract::<Sha256>::new(Some(self.salt.as_ref().unwrap()));
         extract.input_ikm(self.ikm);
@@ -249,7 +275,7 @@ impl KeyMaterial for BlsKeyGen<'_> {
 /// Trait implemented by supported BLS public key types
 pub trait BlsPublicKeyType: 'static {
     /// The concrete key representation
-    type Buffer: Clone + Debug + PartialEq + Sized;
+    type Buffer: Clone + Debug + PartialEq + Sized + Zeroize;
 
     /// The size of the serialized public key
     type BufferSize: ArrayLength<u8>;
@@ -337,7 +363,7 @@ impl BlsPublicKeyType for G2 {
 pub struct G1G2;
 
 impl BlsPublicKeyType for G1G2 {
-    type Buffer = (G1Affine, G2Affine);
+    type Buffer = G1G2Pair;
     type BufferSize = U144;
 
     const ALG_TYPE: BlsCurves = BlsCurves::G1G2;
@@ -355,7 +381,7 @@ impl BlsPublicKeyType for G1G2 {
 
     #[inline]
     fn from_secret_scalar(secret: &Scalar) -> Self::Buffer {
-        (
+        G1G2Pair(
             G1Affine::from(G1Projective::generator() * secret),
             G2Affine::from(G2Projective::generator() * secret),
         )
@@ -370,7 +396,7 @@ impl BlsPublicKeyType for G1G2 {
         let g2: Option<G2Affine> =
             G2Affine::from_compressed(TryInto::<&[u8; 96]>::try_into(&key[48..]).unwrap()).into();
         if let (Some(g1), Some(g2)) = (g1, g2) {
-            Ok((g1, g2))
+            Ok(G1G2Pair(g1, g2))
         } else {
             Err(err_msg!(InvalidKeyData))
         }
@@ -415,6 +441,10 @@ impl From<&BlsKeyPair<G1G2>> for BlsKeyPair<G2> {
     }
 }
 
+#[derive(Clone, Debug, PartialEq, Eq, Zeroize)]
+/// A utility struct combining G1 and G2 public keys
+pub struct G1G2Pair(G1Affine, G2Affine);
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -499,7 +529,7 @@ mod tests {
 
         let jwk = kp.to_jwk_public(None).expect("Error converting key to JWK");
         let jwk = JwkParts::from_str(&jwk).expect("Error parsing JWK");
-        assert_eq!(jwk.kty, "EC");
+        assert_eq!(jwk.kty, JWK_KEY_TYPE);
         assert_eq!(jwk.crv, G1::JWK_CURVE);
         assert_eq!(
             jwk.x,
@@ -509,9 +539,9 @@ mod tests {
         let pk_load = BlsKeyPair::<G1>::from_jwk_parts(jwk).unwrap();
         assert_eq!(kp.to_public_bytes(), pk_load.to_public_bytes());
 
-        let jwk = kp.to_jwk_secret().expect("Error converting key to JWK");
+        let jwk = kp.to_jwk_secret(None).expect("Error converting key to JWK");
         let jwk = JwkParts::from_slice(&jwk).expect("Error parsing JWK");
-        assert_eq!(jwk.kty, "EC");
+        assert_eq!(jwk.kty, JWK_KEY_TYPE);
         assert_eq!(jwk.crv, G1::JWK_CURVE);
         assert_eq!(
             jwk.x,
