@@ -2,7 +2,7 @@
 
 #![no_std]
 #![cfg_attr(docsrs, feature(doc_cfg))]
-// #![deny(missing_docs, missing_debug_implementations, rust_2018_idioms)]
+#![deny(missing_docs, missing_debug_implementations, rust_2018_idioms)]
 
 #[cfg(feature = "alloc")]
 extern crate alloc;
